package com.ruoyi.web.controller.system;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.manager.AsyncManager;
import com.ruoyi.framework.manager.factory.AsyncFactory;
import com.ruoyi.framework.web.service.ThirdLoginService;
import com.ruoyi.framework.web.service.TokenService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.concurrent.TimeUnit;

/**
 * 单点登录Controller
 * @author sgc
 */
@Slf4j
@RestController
public class LoginSsoController {

    @Autowired
    private ThirdLoginService loginService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private RedisCache redisCache;

    @PostMapping("/loginSso")
    public AjaxResult loginSso(@RequestParam("code") String code) {
        System.out.println("token = " + code);
        AjaxResult ajax = AjaxResult.success();

        String token = redisCache.getCacheObject(code);
        LoginUser loginUser = tokenService.getThirdLoginUser(token);

        System.out.println("loginUser = " + loginUser);
        if(loginUser != null){
            ajax.put(Constants.TOKEN, token);
            ajax.put("msg", "登录成功");
            return ajax;
        }
        // 生成令牌(免密登录)
        String tokenNew = loginService.thirdLogin(code);
        if(null==tokenNew){
            ajax = AjaxResult.error("获取认证失败,请重新登入");
        }else {
            redisCache.setCacheObject(code,tokenNew, 60, TimeUnit.MINUTES);
            ajax.put(Constants.TOKEN, tokenNew);
            ajax.put("msg", "登录成功");
        }
        return ajax;

    }


    @PostMapping("/logoutSso")
    public void logoutSso(HttpServletRequest request, HttpServletResponse response,@RequestParam("token") String token)
    {
        String thisToken = redisCache.getCacheObject(token);
        LoginUser loginUser = tokenService.getThirdLoginUser(thisToken);
        if (StringUtils.isNotNull(loginUser))
        {
            String userName = loginUser.getUsername();
            // 删除用户缓存记录
            tokenService.delLoginUser(loginUser.getToken());
            // 记录用户退出日志
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(userName, Constants.LOGOUT, "退出成功"));
        }
        ServletUtils.renderString(response, JSON.toJSONString(AjaxResult.error(HttpStatus.SUCCESS, "退出成功")));
    }

}
