/**
 * Copyright (c) 2016-2019 人人开源 All rights reserved.
 * <p>
 * https://www.renren.io
 * <p>
 * 版权所有，侵权必究！
 */

package com.ruoyi.common.validator;


import com.ruoyi.common.exception.RYException;

import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import java.util.Set;

/**
 * hibernate-validator校验工具类
 * <p>
 * 参考文档：http://docs.jboss.org/hibernate/validator/5.4/reference/en-US/html_single/
 *
 * @author Mark sunlightcs@gmail.com
 */
public class ValidatorUtils {
    private static Validator validator;

    static {
        validator = Validation.buildDefaultValidatorFactory().getValidator();
    }

    /**
     * 校验对象
     *
     * @param object 待校验对象
     * @param groups 待校验的组
     * @throws RYException 校验不通过，则报RYException异常
     */
    public static void validateEntity(Object object, Class<?>... groups)
            throws RYException {
        Set<ConstraintViolation<Object>> constraintViolations = validator.validate(object, groups);
        if (!constraintViolations.isEmpty()) {
            ConstraintViolation<Object> constraint = (ConstraintViolation<Object>) constraintViolations.iterator().next();
            throw new RYException(constraint.getMessage());
        }
    }

    /**
     * 校验对象，版本2
     *
     * @param object 待校验对象
     * @param groups 待校验的组
     * @throws RYException 校验不通过，则报RYException异常
     */
    public static void validateV2(Object object, Class<?>... groups)
            throws RYException {
        validateV2(0, object, groups);
    }

    /**
     * 校验对象，版本2
     *
     * @param i      第几条校验
     * @param object 待校验对象
     * @param groups 待校验的组
     * @throws RYException 校验不通过，则报RYException异常
     */
    public static void validateV2(int i, Object object, Class<?>... groups)
            throws RYException {
        Set<ConstraintViolation<Object>> validateSet = validator.validate(object, groups);
        if (!validateSet.isEmpty()) {
            String messages = validateSet.stream()
                    .map(ConstraintViolation::getMessage)
                    .reduce((m1, m2) -> m1 + "；" + m2)
                    .orElse("参数输入有误！");

            if (i > 0) {
                messages = "第" + i + "条数据：" + messages;
            }
            throw new RYException(messages, 11);
        }
    }
}
