package com.ruoyi.swlscx.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.annotation.DataSource;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.page.R;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.enums.DataSourceType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.swlscx.basic.domain.po.YcExportTask;
import com.ruoyi.swlscx.basic.service.YcExportTaskService;
import com.ruoyi.swlscx.common.PageParams;
import com.ruoyi.swlscx.basic.domain.bo.HyStscABo;
import com.ruoyi.swlscx.basic.domain.po.HyStscA;
import com.ruoyi.swlscx.basic.domain.vo.HyStscAVo;
import com.ruoyi.swlscx.basic.mapper.HyStscAMapper;
import com.ruoyi.swlscx.basic.service.HyStscAService;
import com.ruoyi.swlscx.common.utils.CommonUtils;
import com.ruoyi.swlscx.common.utils.ExcelUtils;
import com.ruoyi.swlscx.common.utils.Query;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

/**
* @author 12974
* @description 针对表【hy_stsc_a】的数据库操作Service实现
* @createDate 2024-07-30 13:37:07
*/
@Service
public class HyStscAServiceImpl extends ServiceImpl<HyStscAMapper, HyStscA>
    implements HyStscAService {

    @Resource
    private ThreadPoolExecutor labelThreadPool;

    @Resource
    private YcExportTaskService ycExportTaskService;

    @Resource
    private ThreadPoolExecutor importThreadPool;

    @Autowired
    private ApplicationContext context;






    @Override
    @Transactional(rollbackFor = Exception.class)
    public void importHyStsc(MultipartFile file) {
        try {
            List<HyStscABo> hyStationBoList = ExcelUtils.readExcel(file.getInputStream(), HyStscABo.class, 2);

            List<HyStscA> hyStationList = hyStationBoList.stream().map(i -> {
                HyStscA hyStscA = new HyStscA();
                BeanUtils.copyProperties(i, hyStscA);
                hyStscA.setDstrvm(Convert.toBigDecimal(i.getDstrvm()));
                hyStscA.setDrar(Convert.toBigDecimal(i.getDrar()));
                hyStscA.setLgtd(Convert.toBigDecimal(i.getLgtd()));
                hyStscA.setLttd(Convert.toBigDecimal(i.getLttd()));
                hyStscA.setEsstmth(Convert.toInt(i.getEsstmth()));
                hyStscA.setEsstyr(Convert.toInt(i.getEsstyr()));
                hyStscA.setWdstmth(Convert.toInt(i.getWdstmth()));
                hyStscA.setEsstmth(Convert.toInt(i.getEsstmth()));
                return hyStscA;
            }).collect(Collectors.toList());
            this.saveBatch(hyStationList, 10000);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 查询站点信息
     */
    @Override
    @DataSource(DataSourceType.MASTER)
    public IPage<HyStscAVo> getStationPageByInfo(PageParams pageParams, String stnm,String stcd , String startTime, String endTime) {
        Map<String, Object> map = CommonUtils.getMonthDataMap(pageParams, startTime, endTime, stcd, stnm);
        return this.baseMapper.selectHyStscAPageByInfo(new Query<HyStscAVo>().getPage(map), map);
    }

    /**
     * 导出测站一览表
     */
    @Override
    public R exportStation(HttpServletResponse response, String startTime, String endTime, String stcd, String stnm) {
        Long userId = SecurityUtils.getUserId();
        String tableName = "测站一览表";
        return taskAndExport(tableName,startTime, endTime, stcd, stnm, userId);
    }

    /**
     * 从mysql导入到sqlserver中
     */
    @Transactional(rollbackFor = Exception.class)
    @DataSource(DataSourceType.SHARDING)
    public void importMysqlToSqlserver(List<HyStscAVo> hyStscAVoList) {
        List<HyStscA> records = hyStscAVoList.stream().map(i->{
            HyStscA hyStscA = new HyStscA();
            BeanUtils.copyProperties(i, hyStscA);
            hyStscA.setDstrvm(Convert.toBigDecimal(i.getDstrvm()));
            hyStscA.setDrar(Convert.toBigDecimal(i.getDrar()));
            hyStscA.setLgtd(Convert.toBigDecimal(i.getLgtd()));
            hyStscA.setLttd(Convert.toBigDecimal(i.getLttd()));
            hyStscA.setEsstmth(Convert.toInt(i.getEsstmth()));
            hyStscA.setEsstyr(Convert.toInt(i.getEsstyr()));
            hyStscA.setWdstmth(Convert.toInt(i.getWdstmth()));
            hyStscA.setEsstmth(Convert.toInt(i.getEsstmth()));
            return hyStscA;
        }).collect(Collectors.toList());
        //先清空临时表的数据
        this.baseMapper.deleteTempData();
        //插入临时表数据
        int batchSize = 50;
        for (int i = 0; i < records.size(); i += batchSize) {
            int end = Math.min(i + batchSize, records.size());
            List<HyStscA> batch = records.subList(i, end);
            // 执行批量插入
            this.baseMapper.insertTempData(batch);
        }
        //从临时表增或修改主表
        this.baseMapper.mergeToData();
    }

    @Override
    public R importDateTosoft(String startTime, String endTime) {
        Long userId = SecurityUtils.getUserId();
        String filename = "测站一览表" + startTime + "-" + endTime+"导入到南方片";
        // 查询是否已存在相同条件且未完成的任务
        List<YcExportTask> ongoingTasks = ycExportTaskService.list(
                new QueryWrapper<YcExportTask>()
                        .eq("user_id", userId)
                        .eq("status", 0) // 状态为0表示任务未完成
                        .like("filename", "测站一览表"));
        if (!ongoingTasks.isEmpty()) {
            return R.error("当前任务正在下载中，请稍后重试");
        }
        YcExportTask ycExportTask = new YcExportTask();
        ycExportTask.setFilename(filename);
        ycExportTask.setUserId(userId);
        ycExportTask.setStartTime(new Date());
        ycExportTask.setStatus(0);
        ycExportTaskService.save(ycExportTask);
        CompletableFuture.runAsync(() -> {
            try {
                IPage<HyStscAVo> stationPageByInfo = this.getStationPageByInfo(new PageParams(0L, 1000000000L), null, null, startTime, endTime);
                HyStscAServiceImpl hyStscAServiceImpl = context.getBean(HyStscAServiceImpl.class);
                hyStscAServiceImpl.importMysqlToSqlserver(stationPageByInfo.getRecords());
                ycExportTask.setStatus(2);
            } catch (Exception e) {
                ycExportTask.setStatus(4);
                log.error(e.getMessage());
                R.error("任务下载失败");
            }finally {
                ycExportTask.setEndTime(new Date());
                ycExportTaskService.updateById(ycExportTask);
            }
        }, importThreadPool);
       return R.ok("任务已提交");
    }

    @Override
    @DataSource(DataSourceType.SHARDING)
    public R testSqlServer() {
        List<HyStscA> list = this.list().subList(0,100);
        return R.ok().put("data",list);
    }


    private R taskAndExport(String tableName,String startTime, String endTime, String stcd, String stnm, Long userId) {
        String startNewTime="";
        String endNewTime="";
        if (ObjectUtils.isNotEmpty(startTime) ){
            startNewTime=startTime.substring(0,10);
        }
        if (ObjectUtils.isNotEmpty(endTime)){
            endNewTime=endTime.substring(0,10);
        }
        String filename = tableName + startNewTime + "-" + endNewTime;
        // 查询是否已存在相同条件且未完成的任务
        List<YcExportTask> ongoingTasks = ycExportTaskService.list(
                new QueryWrapper<YcExportTask>()
                        .eq("user_id", userId)
                        .eq("filename", filename)
                        .eq("status", 0) // 状态为0表示任务未完成
        );

        if (!ongoingTasks.isEmpty()) {
            return R.error("当前任务正在下载中，请稍后重试");
        }
        YcExportTask ycExportTask = new YcExportTask();
        String pathPeffix = RuoYiConfig.getExportPath() +
                File.separator + userId + File.separator+
                DateUtils.parseDateToStr("yyyy-MM-dd",DateUtils.getNowDate()) + File.separator +
                DateUtils.parseDateToStr("HH_mm",DateUtils.getNowDate())+File.separator;
        ycExportTask.setFilename(filename);
        ycExportTask.setUserId(userId);
        ycExportTask.setStartTime(new Date());
        ycExportTask.setStatus(0);
        ycExportTask.setFilepath(pathPeffix);
        ycExportTaskService.save(ycExportTask);
        CompletableFuture.runAsync(() -> {
            try {
                File dest = new File(pathPeffix);
                if (!dest.exists()) {
                    dest.mkdirs();
                }
                String fullPath = pathPeffix + filename;
                exportBlackData(fullPath, startTime, endTime, stcd, stnm);

                ycExportTask.setStatus(1);
                ycExportTask.setEndTime(new Date());
                ycExportTaskService.updateById(ycExportTask);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                R.error("任务下载失败");
            }
        }, labelThreadPool);
        return R.ok("任务已提交，请稍后下载");
    }


    private void exportBlackData(String tableName, String startTime, String endTime, String stcd, String stnm) {
        Map<String, Object> rainMap = CommonUtils.getYearAndDayDataMap(new PageParams(0L, 1000000000L), startTime, endTime, stcd, stnm);
        ExcelUtils.excelBackExport2("station.xls", tableName, (Sheet sheet, CellStyle style) ->
                dataModel1(rainMap, sheet, style));
    }


    public int dataModel1(Map<String, Object> map, Sheet sheet, CellStyle style) {
        // 批量查询数据
        IPage<HyStscAVo> hyStscAVoIPage = this.baseMapper.selectHyStscAPageByInfo(new Query<HyStscAVo>().getPage(map), map);
        List<HyStscAVo> records = hyStscAVoIPage.getRecords();

        // 定义当前sheet的行号
        int rowIndex = 2; // 从第二行开始写入数据
        // 定义全局序号
        int globalIndex = 1; // 从1开始

        // 定义当前sheet的编号
        int sheetIndex = 1;

        // 最大行数限制
        int maxRowsPerSheet = 65536; // 对于 .xls 文件，最大行数为 65536
        int batchSize = 1000; // 每次批量处理1000行，减少逐行写入的开销

        // 复制表头到新sheet的逻辑
        ExcelUtils.copyHeader(sheet, style);

        // 批量处理数据
        for (int i = 0; i < records.size(); i += batchSize) {
            // 处理一批数据
            int toIndex = Math.min(i + batchSize, records.size());
            List<HyStscAVo> batchList = records.subList(i, toIndex);

            // 批量写入数据
            for (HyStscAVo vo : batchList) {
                // 如果当前行数达到最大行数，创建新sheet
                if (rowIndex >= maxRowsPerSheet) {
                    sheetIndex++;
                    sheet = sheet.getWorkbook().createSheet("测站一览表" + sheetIndex);
                    rowIndex = 2; // 重置行号
                    // 复制表头到新sheet
                    ExcelUtils.copyHeader(sheet, style);
                }

                // 创建新行并写入数据
                Row row = ExcelUtils.createRow(sheet, rowIndex);

                // 使用 globalIndex 作为序号
                ExcelUtils.formatCell(row, 0, String.valueOf(globalIndex), style);
                ExcelUtils.formatCell(row, 1, vo.getStcd() != null ? vo.getStcd() : "", style);
                ExcelUtils.formatCell(row, 2, vo.getStnm() != null ? vo.getStnm() : "", style);
                ExcelUtils.formatCell(row, 3, vo.getBshncd() != null ? vo.getBshncd() : "", style);
                ExcelUtils.formatCell(row, 4, vo.getHnnm() != null ? vo.getHnnm() : "", style);
                ExcelUtils.formatCell(row, 5, vo.getRvnm() != null ? vo.getRvnm() : "", style);
                ExcelUtils.formatCell(row, 6, vo.getObitmcd() != null ? vo.getObitmcd() : "", style);
                ExcelUtils.formatCell(row, 7, vo.getAddvcd() != null ? vo.getAddvcd() : "", style);
                ExcelUtils.formatCell(row, 8, vo.getWrrgcd() != null ? vo.getWrrgcd() : "", style);
                ExcelUtils.formatCell(row, 9, vo.getEsstyr() != null ? vo.getEsstyr() : "", style);
                ExcelUtils.formatCell(row, 10, vo.getEsstmth() != null ? vo.getEsstmth() : "", style);
                ExcelUtils.formatCell(row, 11, vo.getWdstyr() != null ? vo.getWdstyr() : "", style);
                ExcelUtils.formatCell(row, 12, vo.getWdstmth() != null ? vo.getWdstmth() : "", style);
                ExcelUtils.formatCell(row, 13, vo.getDrar() != null ? vo.getDrar() : "", style);
                ExcelUtils.formatCell(row, 14, vo.getFlto() != null ? vo.getFlto() : "", style);
                ExcelUtils.formatCell(row, 15, vo.getDstrvm() != null ? vo.getDstrvm() : "", style);
                ExcelUtils.formatCell(row, 16, vo.getAdmag() != null ? vo.getAdmag() : "", style);
                ExcelUtils.formatCell(row, 17, vo.getAdmnst() != null ? vo.getAdmnst() : "", style);
                ExcelUtils.formatCell(row, 18, vo.getStlc() != null ? vo.getStlc() : "", style);
                ExcelUtils.formatCell(row, 19, vo.getLgtd() != null ? CommonUtils.formatNum(vo.getLgtd()) : "", style);
                ExcelUtils.formatCell(row, 20, vo.getLttd() != null ? CommonUtils.formatNum(vo.getLttd()) : "", style);
                ExcelUtils.formatCell(row, 21, vo.getStgrd() != null ? vo.getStgrd() : "", style);
                ExcelUtils.formatCell(row, 22, vo.getFrgrd() != null ? vo.getFrgrd() : "", style);
                ExcelUtils.formatCell(row, 23, vo.getNt() != null ? vo.getNt() : "", style);
                ExcelUtils.formatCell(row, 24, "", style);
                ExcelUtils.formatCell(row, 25, "", style);
                ExcelUtils.formatCell(row, 26, "", style);
                ExcelUtils.formatCell(row, 27, "", style);
                ExcelUtils.formatCell(row, 28, "", style);
                ExcelUtils.formatCell(row, 29, "", style);
                ExcelUtils.formatCell(row, 30, "", style);
                ExcelUtils.formatCell(row, 31, "", style);
                ExcelUtils.formatCell(row, 32, "", style);
                ExcelUtils.formatCell(row, 33, "", style);
                ExcelUtils.formatCell(row, 34, "", style);
                ExcelUtils.formatCell(row, 35, "", style);
                ExcelUtils.formatCell(row, 36, "", style);

                // 增加行号和全局序号
                rowIndex++;
                globalIndex++;
            }
        }
        return 0;
    }







}




