package com.ruoyi.swlscx.basic.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.annotation.DataSource;
import com.ruoyi.common.core.page.R;
import com.ruoyi.common.enums.DataSourceType;
import com.ruoyi.common.utils.Query;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.swlscx.basic.domain.po.YcExportTask;
import com.ruoyi.swlscx.basic.mapper.YcExportTaskMapper;
import com.ruoyi.swlscx.basic.service.YcExportTaskService;
import com.ruoyi.swlscx.common.utils.ExcelUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.List;
import java.util.Map;

/**
 * @author 12974
 * @description 针对表【yc_export_task(导出任务)】的数据库操作Service实现
 * @createDate 2024-10-23 13:57:23
 */
@Service
@Slf4j
public class YcExportTaskServiceImpl extends ServiceImpl<YcExportTaskMapper, YcExportTask>
        implements YcExportTaskService {


    @Override
    public void addTask(HttpServletResponse response, String startTime, String endTime, String stcd, String stnm) {

    }

    @Override
    public R queryPage(Map<String, Object> params) {
        Long userId = SecurityUtils.getUserId();
        params.put("userId", userId);
        IPage<YcExportTask> page = this.baseMapper.queryPage(
                new Query<YcExportTask>().getPage(params),
                params
        );

        return R.ok().put("count", page.getTotal()).put("data", page.getRecords());
    }

    @Override
    public void downloadExcel(HttpServletResponse response, YcExportTask t) {
        ExcelUtils.downLoadExcel(response, t);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public R removeTask(Integer id) {

        //先删除路径下的文件
        YcExportTask task = this.getById(id);
        String fullPath = task.getFilepath()+task.getFilename() + ".xls";
        log.info("fullPath{}",fullPath);
        File file = new File(fullPath);
        if (!file.exists()) {
            throw new RuntimeException("该文件不存在");
        }
        if (file.delete()){
            this.removeById(id);
            return R.ok();
        }
        return R.error("删除失败");
    }

    @Override
    @DataSource(DataSourceType.SHARDING)
    public R getTest() {
       List<Map<String,Object>> list=this.baseMapper.test();
        return R.ok().put("data",list);
    }


    // 删除文件的方法
    private boolean deleteFile(String fullPath) {
        File file = new File(fullPath);
        if (file.exists() && file.isFile()) {
            boolean isDeleted = file.delete();
            log.info("文件删除状态: {}", isDeleted ? "成功" : "失败");
            return isDeleted;
        }
        log.warn("文件不存在或不是文件: {}", fullPath);
        return false;
    }

}




