package com.ruoyi.swlscx.common.utils;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelReader;
import com.alibaba.excel.analysis.ExcelReadExecutor;
import com.alibaba.excel.read.metadata.ReadSheet;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.swlscx.basic.domain.po.YcExportTask;
import com.ruoyi.swlscx.common.functions.ExcelGenerator;
import com.ruoyi.swlscx.common.functions.ExcelGenerator2_1;
import com.ruoyi.swlscx.common.functions.ExcelGenerator2_2;
import com.ruoyi.swlscx.common.functions.ExcelGenerator3;
import com.ruoyi.swlscx.common.listener.ExcelListener;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.core.io.ClassPathResource;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
public class ExcelUtils {

    /**
     * 从指定行开始读取Excel数据
     *
     * @param inputStream   文件输入流
     * @param clazz         映射的实体类
     * @param headRowNumber 从第几行开始读取（从0开始计数）
     * @param <T>           实体类类型
     * @return 实体类对象的列表
     */
    public static <T> List<T> readExcel(InputStream inputStream, Class<T> clazz, int headRowNumber)  {
        // 读取Excel文件
        ExcelReader excelReader = EasyExcel.read(inputStream).build();
        ExcelReadExecutor excelReadExecutor = excelReader.excelExecutor();
        List<ReadSheet> sheets = excelReadExecutor.sheetList();
        List<T> sheetData = new ArrayList<>();
        //读取到的数据
        for (ReadSheet sheet : sheets) {
            ExcelListener<T> listener = new ExcelListener<>();
            ReadSheet readSheet1 = EasyExcel.readSheet(sheet.getSheetNo()).head(clazz).headRowNumber(headRowNumber).registerReadListener(listener).build();
            excelReader.read(readSheet1);
            // 得到sheet名称和其对应的数据
            sheetData.addAll(listener.getData());
        }
        return sheetData;
    }



    /**
     * 导出
     *
     * @param fileName
     * @param response
     * @param workbook
     */
    public static void downLoadExcel(String fileName, HttpServletResponse response, Workbook workbook) {
        try {
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition",
                    "attachment;filename=\"" + URLEncoder.encode(fileName, "UTF-8") + "\"");
            workbook.write(response.getOutputStream());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void downLoadTxt(String fileName, HttpServletResponse response) throws IOException  {
        response.setCharacterEncoding("UTF-8");
        response.setHeader("content-Type", "text/plain");
        response.setHeader("Content-Disposition",
                "attachment;filename=\"" + URLEncoder.encode(fileName, "UTF-8") + "\"");
        OutputStream outputStream = response.getOutputStream();
        byte[] buff = new byte[1024];
        BufferedInputStream bis ;
        ClassPathResource cpr = new ClassPathResource("/excel/" + fileName);
        bis = new BufferedInputStream(cpr.getInputStream());
        int i = bis.read(buff);
        while (i != -1) {
            outputStream.write(buff, 0, buff.length);
            outputStream.flush();
            i = bis.read(buff);
        }
    }

    public static void excelExport(HttpServletResponse response, String templateName, String excelName, ExcelGenerator excelGenerator) {
        try {
            ClassPathResource cpr = new ClassPathResource("/excel/" + templateName);
            InputStream is = cpr.getInputStream();
            Workbook workbook = null;
            if ("xlsx".equals(templateName.split("\\.")[1])) {
                workbook = new XSSFWorkbook(is);
            } else {
                workbook = new HSSFWorkbook(is);
            }

            CellStyle style = workbook.createCellStyle();
            Font font = workbook.createFont();
            font.setFontHeightInPoints((short)9);
            font.setFontName("宋体");
            style.setFont(font);

            Sheet sheet0 = workbook.getSheetAt(0);
            excelGenerator.doSheet(sheet0, style);

            ExcelUtils.downLoadExcel(excelName + "." + templateName.split("\\.")[1], response, workbook);
        }catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void excelExport2(HttpServletResponse response, String templateName, String excelName, ExcelGenerator2_1 excelGenerator) {
        try {
            ClassPathResource cpr = new ClassPathResource("/excel/" + templateName);
            InputStream is = cpr.getInputStream();
            Workbook workbook = null;
            if ("xlsx".equals(templateName.split("\\.")[1])) {
                workbook = new XSSFWorkbook(is);
            } else {
                workbook = new HSSFWorkbook(is);
            }

            CellStyle style = workbook.createCellStyle();
            Font font = workbook.createFont();
            font.setFontHeightInPoints((short)10);
            font.setFontName("宋体");
            style.setFont(font);

            Font font2 = workbook.createFont();
            font2.setFontHeightInPoints((short)12);
            font2.setFontName("宋体");

            CellStyle style1 = workbook.createCellStyle();
            style1.setFont(font2);

            CellStyle style2 = workbook.createCellStyle();
            style2.setAlignment(HorizontalAlignment.LEFT);
            style2.setFont(font);

            CellStyle style3 = workbook.createCellStyle();
            style3.setAlignment(HorizontalAlignment.RIGHT);
            style3.setFont(font);


            Sheet sheet0 = workbook.getSheetAt(0);
            excelGenerator.doSheet(sheet0, style,style1,style2,style3);

            ExcelUtils.downLoadExcel(excelName + "." + templateName.split("\\.")[1], response, workbook);
        }catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void listToExport(HttpServletResponse response, String excelName, List<Map<String, Object>> dataMap, Integer columnNum){
        try {
            ClassPathResource cpr = new ClassPathResource("/excel/" + "list2excel.xls");
            InputStream is = cpr.getInputStream();
            Workbook workbook = new HSSFWorkbook(is);
            Sheet sheet0 = workbook.getSheetAt(0);
            Row row;
            for(int i=0; i< dataMap.size(); i++){
                row=sheet0.createRow(i);
                int j=0;
                for (Map.Entry<String, Object> entry : dataMap.get(i).entrySet()) {
                    Cell cell = row.createCell(j);
                    cell.setCellValue(entry.getValue() == null ? "" : entry.getValue().toString());
                    cell.getCellStyle().setAlignment(HorizontalAlignment.CENTER);
                    j++;
                }
            }
            for(int i=0; i<columnNum; i++){
                sheet0.autoSizeColumn(i,true);
            }
            ExcelUtils.downLoadExcel(excelName + ".xls", response, workbook);
        }catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void excelBackExport(String templateName, String excelName, ExcelGenerator excelGenerator) {
        try {
            ClassPathResource cpr = new ClassPathResource("/excel/" + templateName);
            InputStream is = cpr.getInputStream();
            Workbook workbook = null;
            if ("xlsx".equals(templateName.split("\\.")[1])) {
                workbook = new XSSFWorkbook(is);
            } else {
                workbook = new HSSFWorkbook(is);
            }

            CellStyle style = workbook.createCellStyle();
            Font font = workbook.createFont();
            font.setFontHeightInPoints((short)9);
            font.setFontName("宋体");
            style.setFont(font);

            Sheet sheet0 = workbook.getSheetAt(0);
            excelGenerator.doSheet(sheet0, style);

            ExcelUtils.downLoadBackExcel(excelName + "." + templateName.split("\\.")[1], workbook);
        }catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void excelBackExport2(String templateName, String excelName, ExcelGenerator3 excelGenerator) {
        try {
            ClassPathResource cpr = new ClassPathResource("/excel/" + templateName);
            InputStream is = cpr.getInputStream();
            Workbook workbook = null;
            if ("xlsx".equals(templateName.split("\\.")[1])) {
                workbook = new XSSFWorkbook(is);
            } else {
                workbook = new HSSFWorkbook(is);
            }

            CellStyle style = workbook.createCellStyle();
            Font font = workbook.createFont();
            font.setFontHeightInPoints((short)9);
            font.setFontName("宋体");
            style.setFont(font);

            Sheet sheet0 = workbook.getSheetAt(0);
            int res = excelGenerator.doSheet(sheet0, style);
            System.out.println("excelBackExport2  res==>" + res);
            if (res == 0){
                ExcelUtils.downLoadBackExcel(excelName + "." + templateName.split("\\.")[1], workbook);
            }
        }catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 逐潮高低潮表专用
     */
    public static void excelExportTide1Back(String templateName, String excelName, ExcelGenerator2_2 excelGenerator) {
        try {
            ClassPathResource cpr = new ClassPathResource("/excel/" + templateName);
            InputStream is = cpr.getInputStream();
            Workbook workbook = null;
            if ("xlsx".equals(templateName.split("\\.")[1])) {
                workbook = new XSSFWorkbook(is);
            } else {
                workbook = new HSSFWorkbook(is);
            }

            CellStyle style = workbook.createCellStyle();
            Font font = workbook.createFont();
            font.setFontHeightInPoints((short)10);
            font.setFontName("宋体");
            style.setFont(font);

            Font font2 = workbook.createFont();
            font2.setFontHeightInPoints((short)12);
            font2.setFontName("宋体");

            CellStyle style1 = workbook.createCellStyle();
            style1.setFont(font2);

            CellStyle style2 = workbook.createCellStyle();
            style2.setAlignment(HorizontalAlignment.LEFT);
            style2.setFont(font);

            CellStyle style3 = workbook.createCellStyle();
            style3.setAlignment(HorizontalAlignment.RIGHT);
            style3.setFont(font);


            Sheet sheet0 = workbook.getSheetAt(0);
            int res = excelGenerator.doSheet(sheet0, style,style1,style2,style3);
            if (res == 0) {
                ExcelUtils.downLoadBackExcel(excelName + "." + templateName.split("\\.")[1], workbook);
            }
        }catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void downLoadBackExcel(String fileName,  Workbook workbook) {
        try {
            FileOutputStream fileOut = new FileOutputStream(fileName);
            workbook.write(fileOut);
            fileOut.close();
            workbook.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void formatCell(Row row, int index, String value, CellStyle style) {
        Cell cell = row.createCell(index);
        cell.setCellValue(value);
        // 对齐方式
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        // 设置上下左右边框
        style.setBorderLeft(BorderStyle.THIN);
        style.setBorderRight(BorderStyle.THIN);
        style.setBorderTop(BorderStyle.THIN);    // 添加顶部边框
        style.setBorderBottom(BorderStyle.THIN); // 添加底部边框
        // 应用样式
        cell.setCellStyle(style);
    }


    public static Row createRow(Sheet sheet, int index) {
        if (sheet.getRow(index) == null) {
            return sheet.createRow(index);
        } else {
            return sheet.getRow(index);
        }
    }


    public static void downLoadExcel(HttpServletResponse response, YcExportTask t) {
        Long userId = SecurityUtils.getUserId();
        String fullPath = t.getFilepath()+t.getFilename() + ".xls";
        log.info("fullPath{}",fullPath);
        File file = new File(fullPath);
        if (!file.exists()) {
            throw new RuntimeException("该文件不存在");
        }

        try (InputStream in = Files.newInputStream(file.toPath());
             OutputStream out = response.getOutputStream()) {
            // 设置响应头，通知浏览器以附件形式下载
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition",
                    "attachment;filename=\"" + URLEncoder.encode(t.getFilename(), "UTF-8") + "\"");
            byte[] buffer = new byte[1024];
            int len;
            while ((len = in.read(buffer)) != -1) {
                out.write(buffer, 0, len);
            }
            out.flush(); // 确保所有数据写入响应
        } catch (IOException e) {
            throw new RuntimeException("文件下载失败", e);
        }
    }



    public static void copyHeader(Sheet targetSheet, CellStyle style) {
        // 获取第一个 sheet
        Sheet sourceSheet = targetSheet.getWorkbook().getSheetAt(0); // 假设源 sheet 是第一个

        // 复制第一行（表头）
        Row headerRow1 = sourceSheet.getRow(0); // 第1行
        Row newHeaderRow1 = targetSheet.createRow(0); // 在新sheet中创建第一行
        copyRow(headerRow1, newHeaderRow1);

        // 复制第二行
        Row headerRow2 = sourceSheet.getRow(1); // 第2行
        Row newHeaderRow2 = targetSheet.createRow(1); // 在新sheet中创建第二行
        copyRow(headerRow2, newHeaderRow2);

        // 复制合并单元格
        int mergedRegionsCount = sourceSheet.getNumMergedRegions();
        for (int i = 0; i < mergedRegionsCount; i++) {
            CellRangeAddress mergedRegion = sourceSheet.getMergedRegion(i);
            // 检查合并区域是否在需要复制的行范围内
            if (mergedRegion.getFirstRow() <= 1 && mergedRegion.getLastRow() >= 0) {
                // 检查目标 sheet 中是否已存在相同的合并区域
                boolean exists = false;
                for (int j = 0; j < targetSheet.getNumMergedRegions(); j++) {
                    CellRangeAddress existingRegion = targetSheet.getMergedRegion(j);
                    if (existingRegion.equals(mergedRegion)) {
                        exists = true;
                        break;
                    }
                }

                // 如果不存在，则添加合并区域
                if (!exists) {
                    targetSheet.addMergedRegion(new CellRangeAddress(
                            mergedRegion.getFirstRow(), mergedRegion.getLastRow(),
                            mergedRegion.getFirstColumn(), mergedRegion.getLastColumn()
                    ));
                }
            }
        }
    }

    // 假设 copyRow 方法是已定义的方法，用于复制行的内容和样式
    public static void copyRow(Row sourceRow, Row targetRow) {
        for (int i = 0; i < sourceRow.getPhysicalNumberOfCells(); i++) {
            Cell sourceCell = sourceRow.getCell(i);
            Cell targetCell = targetRow.createCell(i);
            if (sourceCell != null) {
                // 复制单元格内容和样式
                targetCell.setCellValue(sourceCell.getStringCellValue());
                targetCell.setCellStyle(sourceCell.getCellStyle());
            }
        }
    }


}
