package com.ruoyi.swlscx.day.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ruoyi.common.core.page.R;
import com.ruoyi.swlscx.common.PageParams;
import com.ruoyi.swlscx.day.domain.vo.*;
import com.ruoyi.swlscx.day.service.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;

/**
 * @Author al
 * @Date 2024/10/17 15:16
 * @Description: TODO
 * @Version
 */
@RestController
@RequestMapping("/day")
@RequiredArgsConstructor
@Slf4j
public class DayTableController {

    private final HyDpCService hyDpCService;

    private final HyDweCService hyDweCService;

    private final HyDqCService hyDqCService;

    private final HyDzCService hyDzCService;

    private final HyDcsFService hyDcsFService;

    private final HyDwtCService hyDwtCService;

    /**
     * 获取日降雨量表数据
     */
    @GetMapping("/getDayRainList")
    public R getRainList(@RequestParam("pageNum") Integer pageNum,
                     @RequestParam("pageSize") Integer pageSize,
                     @RequestParam("startTime") String startTime,
                     @RequestParam("endTime") String endTime,
                     @RequestParam("stcd") String stcd,
                     @RequestParam("stnm") String stnm){
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyDpCVo> hyDpCVoIPage = hyDpCService.selectHyDpCDataByPageAndInfo(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", hyDpCVoIPage.getRecords()).put("count", hyDpCVoIPage.getTotal());
    }

    /**
     * 查询日水面蒸发表数据
     */
    @GetMapping("/getEvaporationWaterList")
    public R getEvaporationWaterList(@RequestParam("pageNum") Integer pageNum,
                     @RequestParam("pageSize") Integer pageSize,
                     @RequestParam("startTime") String startTime,
                     @RequestParam("endTime") String endTime,
                     @RequestParam("stcd") String stcd,
                     @RequestParam("stnm") String stnm){
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyDweCVo> hyDweCVoIPage = hyDweCService.selectHyDweCDataByPageAndInfo(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", hyDweCVoIPage.getRecords()).put("count", hyDweCVoIPage.getTotal());
    }

    /**
     * 查询日流量表数据
     */
    @GetMapping("/getDayFlowList")
    public R getDayFlowList(@RequestParam("pageNum") Integer pageNum,
                               @RequestParam("pageSize") Integer pageSize,
                               @RequestParam("startTime") String startTime,
                               @RequestParam("endTime") String endTime,
                               @RequestParam("stcd") String stcd,
                               @RequestParam("stnm") String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyDqCVo> hyDqCVoIPage = hyDqCService.selectHyDqCDataByPageAndInfo(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", hyDqCVoIPage.getRecords()).put("count", hyDqCVoIPage.getTotal());
    }

    /**查询日水位表数据**/
    @GetMapping("/getDayWaterLeverList")
    public R getWaterLeverList(@RequestParam("pageNum") Integer pageNum,
                               @RequestParam("pageSize") Integer pageSize,
                               @RequestParam("startTime") String startTime,
                               @RequestParam("endTime") String endTime,
                               @RequestParam("stcd") String stcd,
                               @RequestParam("stnm") String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyDzCVo> hyDzCVoIPage = hyDzCService.selectHyDzCPageByInfo(pageParams, startTime, endTime, stnm, stcd);
        return R.ok().put("data", hyDzCVoIPage.getRecords()).put("count", hyDzCVoIPage.getTotal());
    }


    /**查询日平均含沙量表表数据**/
    @GetMapping("/getDaySedimentConcentrationList")
    public R getDaySedimentConcentrationList(@RequestParam("pageNum") Integer pageNum,
                               @RequestParam("pageSize") Integer pageSize,
                               @RequestParam("startTime") String startTime,
                               @RequestParam("endTime") String endTime,
                               @RequestParam("stcd") String stcd,
                               @RequestParam("stnm") String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyDcsFVo> hyDcsFVoIPage = hyDcsFService.selectHyDcsFPageByInfo(pageParams, startTime, endTime, stnm, stcd);
        return R.ok().put("data", hyDcsFVoIPage.getRecords()).put("count", hyDcsFVoIPage.getTotal());
    }


    /**查询日水温表数据**/
    @GetMapping("/getDayWaterTemperatureList")
    public R getDayWaterTemperatureList(@RequestParam("pageNum") Integer pageNum,
                                             @RequestParam("pageSize") Integer pageSize,
                                             @RequestParam("startTime") String startTime,
                                             @RequestParam("endTime") String endTime,
                                             @RequestParam("stcd") String stcd,
                                             @RequestParam("stnm") String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyDwtCVo> hyDwtCVoIPage = hyDwtCService.selectHyDwtCPageByInfo(pageParams, startTime, endTime, stnm, stcd);
        return R.ok().put("data", hyDwtCVoIPage.getRecords()).put("count", hyDwtCVoIPage.getTotal());
    }

}
