package com.ruoyi.swlscx.day.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.annotation.DataSource;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.page.R;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.enums.DataSourceType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.swlscx.basic.domain.po.YcExportTask;
import com.ruoyi.swlscx.basic.service.YcExportTaskService;
import com.ruoyi.swlscx.common.PageParams;
import com.ruoyi.swlscx.common.constants.SystemConstants;
import com.ruoyi.swlscx.common.utils.CommonUtils;
import com.ruoyi.swlscx.common.utils.ExcelUtils;
import com.ruoyi.swlscx.common.utils.Query;
import com.ruoyi.swlscx.day.domain.bo.HyDwtCBo;
import com.ruoyi.swlscx.day.domain.po.HyDwtC;
import com.ruoyi.swlscx.day.domain.vo.HyDweCVo;
import com.ruoyi.swlscx.day.domain.vo.HyDwtCVo;
import com.ruoyi.swlscx.day.service.HyDwtCService;
import com.ruoyi.swlscx.day.mapper.HyDwtCMapper;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

/**
* @author 12974
* @description 针对表【hy_dwt_c(日水温表)】的数据库操作Service实现
* @createDate 2024-10-17 15:07:12
*/
@Service
public class HyDwtCServiceImpl extends ServiceImpl<HyDwtCMapper, HyDwtC>
    implements HyDwtCService{

    @Resource
    private ThreadPoolExecutor labelThreadPool;

    @Resource
    private YcExportTaskService ycExportTaskService;

    @Autowired
    private ApplicationContext context;

    @Resource
    private ThreadPoolExecutor importThreadPool;

    /**
     * 导入日水温表数据
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void importHyDwtCData(MultipartFile file) {
        try {
            List<HyDwtCBo> hyDwtCBos = ExcelUtils.readExcel(file.getInputStream(),
                    HyDwtCBo.class, 2);
            List<HyDwtC> hyDwtCList = hyDwtCBos.stream().map(i -> {
                HyDwtC hyDwtC = new HyDwtC();
                BeanUtils.copyProperties(i, hyDwtC);
                hyDwtC.setDt(DateUtils.parseDate(i.getDt()));
                hyDwtC.setWtmp(Convert.toBigDecimal(i.getWtmp()));
                return hyDwtC;
            }).collect(Collectors.toList());
            this.saveBatch(hyDwtCList, 10000);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 查询日水温表数据
     */
    @Override
    public IPage<HyDwtCVo> selectHyDwtCPageByInfo(PageParams pageParams, String startTime, String endTime, String stnm, String stcd) {
        Map<String, Object> map = CommonUtils.getYearAndDayDataMap(pageParams, startTime, endTime, stcd, stnm);
        return this.baseMapper.selectHyDwtCPageByInfo(new Query<HyDwtCVo>().getPage(map), map);

    }

    /**
     * 从mysql导入到sqlserver中
     */
    @Transactional(rollbackFor = Exception.class)
    @DataSource(DataSourceType.SHARDING)
    public void importMysqlToSqlserver(List<HyDwtCVo> hyDwtCVoList) {
        List<HyDwtC> records = hyDwtCVoList.stream().map(i->{
            HyDwtC hyDwtC = new HyDwtC();
            com.ruoyi.common.utils.bean.BeanUtils.copyProperties(i, hyDwtC);
            return hyDwtC;
        }).collect(Collectors.toList());
        this.saveBatch(records,10000);
    }


    @Override
    public R exportDayWaterTemperature(HttpServletResponse response, String startTime, String endTime, String stcd, String stnm) {
        Long userId = SecurityUtils.getUserId();
        String tableName = "日水温表";
        return taskAndExport(tableName,startTime, endTime, stcd, stnm, userId);

//        Map<String, Object> dataMap = CommonUtils.getYearAndDayDataMap(new PageParams(0L, 100000L), startTime, endTime, stcd, stnm);
//        ExcelUtils.excelExport(response, "dayWaterTemperature.xls", "日水温表", (Sheet sheet, CellStyle style) -> {
//            dataModel(dataMap, sheet, style);
//        });
    }

    @Override
    public R importDateTosoft(String startTime, String endTime) {
        Long userId = SecurityUtils.getUserId();
        String filename = "日水温表" + startTime + "-" + endTime+"导入到南方片";
        YcExportTask ycExportTask = new YcExportTask();
        ycExportTask.setFilename(filename);
        ycExportTask.setUserId(userId);
        ycExportTask.setStartTime(new Date());
        ycExportTask.setStatus(0);
        ycExportTaskService.save(ycExportTask);
        CompletableFuture.runAsync(() -> {
            try {
                IPage<HyDwtCVo> stationPageByInfo = this.selectHyDwtCPageByInfo(new PageParams(0L, 1000000000L), startTime, endTime, null, null);
                HyDwtCServiceImpl hyDwtCService = context.getBean(HyDwtCServiceImpl.class);
                hyDwtCService.importMysqlToSqlserver(stationPageByInfo.getRecords());
                ycExportTask.setStatus(2);
                ycExportTask.setEndTime(new Date());
                ycExportTaskService.updateById(ycExportTask);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                ycExportTask.setStatus(4);
                ycExportTask.setEndTime(new Date());
                ycExportTaskService.updateById(ycExportTask);
            }
        }, importThreadPool);
        return R.ok("任务已提交");
    }

    private R taskAndExport(String tableName,String startTime, String endTime, String stcd, String stnm, Long userId) {
        String startNewTime="";
        String endNewTime="";
        if (ObjectUtils.isNotEmpty(startTime) ){
            startNewTime=startTime.substring(0,10);
        }
        if (ObjectUtils.isNotEmpty(endTime)){
            endNewTime=endTime.substring(0,10);
        }
        String filename = tableName + startNewTime + "-" + endNewTime;
        // 查询是否已存在相同条件且未完成的任务
        List<YcExportTask> ongoingTasks = ycExportTaskService.list(
                new QueryWrapper<YcExportTask>()
                        .eq("user_id", userId)
                        .eq("filename", filename)
                        .eq("status", 0) // 状态为0表示任务未完成
        );

        if (!ongoingTasks.isEmpty()) {
            return R.error("当前任务正在下载中，请稍后重试");
        }

        YcExportTask ycExportTask = new YcExportTask();
        String pathPeffix = RuoYiConfig.getExportPath() +
                File.separator + userId + File.separator+
                DateUtils.parseDateToStr("yyyy-MM-dd",DateUtils.getNowDate()) + File.separator +
                DateUtils.parseDateToStr("HH_mm",DateUtils.getNowDate())+File.separator;
        ycExportTask.setFilename(filename);
        ycExportTask.setUserId(userId);
        ycExportTask.setStartTime(new Date());
        ycExportTask.setStatus(0);
        ycExportTask.setFilepath(pathPeffix);
        ycExportTaskService.save(ycExportTask);
        CompletableFuture.runAsync(() -> {
            try {
                File dest = new File(pathPeffix);
                if (!dest.exists()) {
                    dest.mkdirs();
                }
                String fullPath = pathPeffix + filename;
                exportBlackData(fullPath, startTime, endTime, stcd, stnm);

                ycExportTask.setStatus(1);
                ycExportTask.setEndTime(new Date());
                ycExportTaskService.updateById(ycExportTask);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                R.error("任务下载失败");
            }
        }, labelThreadPool);
        return R.ok("任务已提交，请稍后下载");
    }

    private void exportBlackData(String tableName, String startTime, String endTime, String stcd, String stnm) {
        Map<String, Object> rainMap = CommonUtils.getYearAndDayDataMap(new PageParams(0L, 1000000000L), startTime, endTime, stcd, stnm);
        ExcelUtils.excelBackExport2("dayWaterTemperature.xls", tableName, (Sheet sheet, CellStyle style) ->
                dataModel1(rainMap, sheet, style));
    }


    public int dataModel1(Map<String, Object> map, Sheet sheet, CellStyle style) {
        // 批量查询数据
        IPage<HyDwtCVo> hyDweCVoIPage = this.baseMapper.selectHyDwtCPageByInfo(new Query<HyDwtCVo>().getPage(map), map);
        List<HyDwtCVo> records = hyDweCVoIPage.getRecords();
        // 定义当前sheet的行号
        int rowIndex = 2; // 从第二行开始写入数据
        // 定义全局序号
        int globalIndex = 1; // 从1开始

        // 定义当前sheet的编号
        int sheetIndex = 1;

        // 最大行数限制
        int maxRowsPerSheet = 65536; // 对于 .xls 文件，最大行数为 65536
        int batchSize = 1000; // 每次批量处理1000行，减少逐行写入的开销

        // 复制表头到新sheet的逻辑
        ExcelUtils.copyHeader(sheet, style);

        // 批量处理数据
        for (int i = 0; i < records.size(); i += batchSize) {
            // 处理一批数据
            int toIndex = Math.min(i + batchSize, records.size());
            List<HyDwtCVo> batchList = records.subList(i, toIndex);

            // 批量写入数据
            for (HyDwtCVo vo : batchList) {
                // 如果当前行数达到最大行数，创建新sheet
                if (rowIndex >= maxRowsPerSheet) {
                    sheetIndex++;
                    sheet = sheet.getWorkbook().createSheet("日水温表" + sheetIndex);
                    rowIndex = 2; // 重置行号
                    // 复制表头到新sheet
                    ExcelUtils.copyHeader(sheet, style);
                }

                // 创建新行并写入数据
                Row row = ExcelUtils.createRow(sheet, rowIndex);

                // 使用 globalIndex 作为序号
                ExcelUtils.formatCell(row, 0, String.valueOf(globalIndex), style);
                ExcelUtils.formatCell(row, 1, vo.getStcd() != null ? vo.getStcd() : "", style);
                ExcelUtils.formatCell(row, 2, vo.getStnm() != null ? vo.getStnm() : "", style);
                ExcelUtils.formatCell(row, 3, vo.getDt() != null ? DateUtils.parseDateToStr(SystemConstants.TIME_FORMAT,vo.getDt()) : "", style);
                ExcelUtils.formatCell(row, 4, vo.getWtmp() != null ? CommonUtils.formatNum(vo.getWtmp()) : "", style);
                ExcelUtils.formatCell(row, 5, vo.getWtmprcd() != null ? vo.getWtmprcd() : "", style);
                // 增加行号和全局序号
                rowIndex++;
                globalIndex++;
            }
        }
        return 0;
    }

    public void dataModel(Map<String, Object> map, Sheet sheet, CellStyle style) {
        // 批量查询数据
        IPage<HyDwtCVo> hyDweCVoIPage = this.baseMapper.selectHyDwtCPageByInfo(new Query<HyDwtCVo>().getPage(map), map);
        List<HyDwtCVo> records = hyDweCVoIPage.getRecords();
        // 定义当前sheet的行号
        int rowIndex = 2;
        // 定义当前sheet的编号
        int sheetIndex = 1;
        Row row;
        // 最大行数限制
        int maxRowsPerSheet = 65536; // 对于 .xls 文件，最大行数为 65536
        int batchSize = 1000; // 每次批量处理1000行，减少逐行写入的开销
        // 批量处理数据
        for (int i = 0; i < records.size(); i += batchSize) {
            // 处理一批数据
            int toIndex = Math.min(i + batchSize, records.size());
            List<HyDwtCVo> batchList = records.subList(i, toIndex);
            // 批量写入数据
            for (HyDwtCVo vo : batchList) {
                // 如果当前行数达到最大行数，创建新sheet
                if (rowIndex >= maxRowsPerSheet) {
                    sheetIndex++;
                    sheet = sheet.getWorkbook().createSheet("日水温表_" + sheetIndex);
                    rowIndex = 2; // 重置行号
                }
                // 创建新行并写入数据
                row = ExcelUtils.createRow(sheet, rowIndex);
                //序号
                ExcelUtils.formatCell(row, 0, String.valueOf(rowIndex - 1), style);
                ExcelUtils.formatCell(row, 1, vo.getStcd() != null ? vo.getStcd() : "", style);
                ExcelUtils.formatCell(row, 2, vo.getStnm() != null ? vo.getStnm() : "", style);
                ExcelUtils.formatCell(row, 3, vo.getDt() != null ? DateUtils.parseDateToStr(SystemConstants.TIME_FORMAT,vo.getDt()) : "", style);
                ExcelUtils.formatCell(row, 4, vo.getWtmp() != null ? CommonUtils.formatNum(vo.getWtmp()) : "", style);
                ExcelUtils.formatCell(row, 5, vo.getWtmprcd() != null ? vo.getWtmprcd() : "", style);
                rowIndex++;
            }
        }
    }
}




