package com.ruoyi.swlscx.month.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.annotation.DataSource;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.page.R;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.enums.DataSourceType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.swlscx.basic.domain.po.YcExportTask;
import com.ruoyi.swlscx.basic.service.YcExportTaskService;
import com.ruoyi.swlscx.common.PageParams;
import com.ruoyi.swlscx.month.domain.bo.HyMtqsEBo;
import com.ruoyi.swlscx.month.domain.po.HyMtqsE;
import com.ruoyi.swlscx.month.domain.po.HyMtweE;
import com.ruoyi.swlscx.month.domain.vo.HyMtqsEVo;
import com.ruoyi.swlscx.month.domain.vo.HyMtweEVo;
import com.ruoyi.swlscx.month.mapper.HyMtqsEMapper;
import com.ruoyi.swlscx.month.service.HyMtqsEService;
import com.ruoyi.swlscx.common.utils.CommonUtils;
import com.ruoyi.swlscx.common.utils.ExcelUtils;
import com.ruoyi.swlscx.common.utils.Query;
import com.ruoyi.swlscx.year.domain.vo.HyHmxpFVo;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

/**
* @author 12974
* @description 针对表【hy_mtqs_e(月输沙率表)】的数据库操作Service实现
* @createDate 2024-08-05 16:03:07
*/
@Service
public class HyMtqsEServiceImpl extends ServiceImpl<HyMtqsEMapper, HyMtqsE>
    implements HyMtqsEService{

    @Resource
    private ThreadPoolExecutor labelThreadPool;

    @Resource
    private YcExportTaskService ycExportTaskService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void importHyMtqsEData(MultipartFile file) {
        try {
            List<HyMtqsEBo> hyMtqsEBos = ExcelUtils.readExcel(file.getInputStream(), HyMtqsEBo.class, 2);
            // 获取数据
            List<HyMtqsE> dataList = hyMtqsEBos.parallelStream().map(i -> {
                HyMtqsE hyMtqE = new HyMtqsE();
                BeanUtils.copyProperties(i, hyMtqE);
                hyMtqE.setYr(Convert.toInt(i.getYr()));
                hyMtqE.setMth(Convert.toInt(i.getMth()));
                hyMtqE.setAvqs(Convert.toBigDecimal(i.getAvqs()));
                hyMtqE.setMxdyqs(Convert.toBigDecimal(i.getMxdyqs()));
                hyMtqE.setMxdyqsodt(DateUtils.parseDate(i.getMxdyqsodt()));
                return hyMtqE;
            }).collect(Collectors.toList());
            this.saveBatch(dataList,10000);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 查询月输沙率表数据
     */
    @Override
    public IPage<HyMtqsEVo> selectHyMtqsEDataByPageAndInfo(PageParams pageParams, String stcdIds, String startTime, String endTime,String stcd,String stnm) {
        Map<String, Object> map = CommonUtils.getMonthDataMap(pageParams, startTime, endTime, stcd, stnm);
        return this.baseMapper.selectHyMtqsEByPageAndInfo(new Query<HyMtqsEVo>().getPage(map), map);
    }

    /**
     * 导出月输沙率表
     */
    @Override
    public R exportMonthSedimentDischargeList(HttpServletResponse response, String startTime, String endTime, String stcd, String stnm) {

        Long userId = SecurityUtils.getUserId();
        String tableName = "月输沙率表";
        return taskAndExport(tableName,startTime, endTime, stcd, stnm, userId);

//        Map<String, Object> map = CommonUtils.getMonthDataMap(new PageParams(0L,100000L), startTime, endTime, stcd, stnm);
//        ExcelUtils.excelExport(response,"momthSedimentTransportRate.xls","月输沙率表",(Sheet sheet, CellStyle style) ->{
//            dataModel(map, sheet, style);
//        });
    }


    @Resource
    private ThreadPoolExecutor importThreadPool;

    @Autowired
    private ApplicationContext context;

    /**
     * 从mysql导入到sqlserver中
     */
    @Transactional(rollbackFor = Exception.class)
    @DataSource(DataSourceType.SHARDING)
    public void importMysqlToSqlserver(List<HyMtqsEVo> hyMtqsEVoList) {
        List<HyMtqsE> records = hyMtqsEVoList.stream().map(i->{
            HyMtqsE hyMtqE = new HyMtqsE();
            BeanUtils.copyProperties(i, hyMtqE);
            hyMtqE.setYr(Convert.toInt(i.getYr()));
            hyMtqE.setMth(Convert.toInt(i.getMth()));
            hyMtqE.setAvqs(Convert.toBigDecimal(i.getAvqs()));
            hyMtqE.setMxdyqs(Convert.toBigDecimal(i.getMxdyqs()));
            hyMtqE.setMxdyqsodt(DateUtils.parseDate(i.getMxdyqsodt()));
            return hyMtqE;
        }).collect(Collectors.toList());
        //先清空临时表的数据
        this.baseMapper.deleteTempData();
        //插入临时表数据
        int batchSize = 50;
        for (int i = 0; i < records.size(); i += batchSize) {
            int end = Math.min(i + batchSize, records.size());
            List<HyMtqsE> batch = records.subList(i, end);
            // 执行批量插入
            this.baseMapper.insertTempData(batch);
        }
        //从临时表增或修改主表
        this.baseMapper.mergeToData();
    }

    @Override
    public R importDateTosoft(String startTime, String endTime) {
        Long userId = SecurityUtils.getUserId();
        String filename = "月输沙率表" + startTime + "-" + endTime+"导入到南方片";
        // 查询是否已存在相同条件且未完成的任务
        List<YcExportTask> ongoingTasks = ycExportTaskService.list(
                new QueryWrapper<YcExportTask>()
                        .eq("user_id", userId)
                        .eq("status", 0) // 状态为0表示任务未完成
                        .like("filename", "月输沙率表"));
        if (!ongoingTasks.isEmpty()) {
            return R.error("当前任务正在下载中，请稍后重试");
        }
        YcExportTask ycExportTask = new YcExportTask();
        ycExportTask.setFilename(filename);
        ycExportTask.setUserId(userId);
        ycExportTask.setStartTime(new Date());
        ycExportTask.setStatus(0);
        ycExportTaskService.save(ycExportTask);
        CompletableFuture.runAsync(() -> {
            try {
                IPage<HyMtqsEVo> pageByInfo = this.selectHyMtqsEDataByPageAndInfo(new PageParams(0L, 1000000000L), null, startTime, endTime, null,null);
//                IPage<HyHmxpFVo> pageByInfo = this.selectMaxHRainListPageByInfo(new PageParams(0L, 1000000000L), null, null, startTime, endTime);
                HyMtqsEServiceImpl hyMtqsEService = context.getBean(HyMtqsEServiceImpl.class);
                hyMtqsEService.importMysqlToSqlserver(pageByInfo.getRecords());
                ycExportTask.setStatus(2);
            } catch (Exception e) {
                ycExportTask.setStatus(4);
                log.error(e.getMessage());
                R.error("任务下载失败");
            }finally {
                ycExportTask.setEndTime(new Date());
                ycExportTaskService.updateById(ycExportTask);
            }
        }, importThreadPool);
        return R.ok("任务已提交");
    }

    private R taskAndExport(String tableName,String startTime, String endTime, String stcd, String stnm, Long userId) {
        String startNewTime="";
        String endNewTime="";
        if (ObjectUtils.isNotEmpty(startTime) ){
            startNewTime=startTime.substring(0,6);
        }
        if (ObjectUtils.isNotEmpty(endTime)){
            endNewTime=endTime.substring(0,6);
        }
        String filename = tableName + startNewTime + "-" + endNewTime;
        // 查询是否已存在相同条件且未完成的任务
        List<YcExportTask> ongoingTasks = ycExportTaskService.list(
                new QueryWrapper<YcExportTask>()
                        .eq("user_id", userId)
                        .eq("filename", filename)
                        .eq("status", 0) // 状态为0表示任务未完成
        );

        if (!ongoingTasks.isEmpty()) {
            return R.error("当前任务正在下载中，请稍后重试");
        }

        YcExportTask ycExportTask = new YcExportTask();
        String pathPeffix = RuoYiConfig.getExportPath() +
                File.separator + userId + File.separator+
                DateUtils.parseDateToStr("yyyy-MM-dd",DateUtils.getNowDate()) + File.separator +
                DateUtils.parseDateToStr("HH_mm",DateUtils.getNowDate())+File.separator;
        ycExportTask.setFilename(filename);
        ycExportTask.setUserId(userId);
        ycExportTask.setStartTime(new Date());
        ycExportTask.setStatus(0);
        ycExportTask.setFilepath(pathPeffix);
        ycExportTaskService.save(ycExportTask);
        CompletableFuture.runAsync(() -> {
            try {
                File dest = new File(pathPeffix);
                if (!dest.exists()) {
                    dest.mkdirs();
                }
                String fullPath = pathPeffix + filename;
                exportBlackData(fullPath, startTime, endTime, stcd, stnm);

                ycExportTask.setStatus(1);
                ycExportTask.setEndTime(new Date());
                ycExportTaskService.updateById(ycExportTask);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                R.error("任务下载失败");
            }
        }, labelThreadPool);
        return R.ok("任务已提交，请稍后下载");
    }

    private void exportBlackData(String tableName, String startTime, String endTime, String stcd, String stnm) {
        Map<String, Object> rainMap = CommonUtils.getMonthDataMap(new PageParams(0L, 1000000000L), startTime, endTime, stcd, stnm);
        ExcelUtils.excelBackExport2("momthSedimentTransportRate.xls", tableName, (Sheet sheet, CellStyle style) ->
                dataModel1(rainMap, sheet, style));
    }

    public int dataModel1(Map<String, Object> map, Sheet sheet, CellStyle style) {
        // 批量查询数据
        IPage<HyMtqsEVo> hyMtqsEVoIPage = this.baseMapper.selectHyMtqsEByPageAndInfo(new Query<HyMtqsEVo>().getPage(map), map);
        List<HyMtqsEVo> records = hyMtqsEVoIPage.getRecords();

        // 定义当前sheet的行号
        int rowIndex = 2; // 从第二行开始写入数据
        // 定义全局序号
        int globalIndex = 1; // 从1开始

        // 定义当前sheet的编号
        int sheetIndex = 1;

        // 最大行数限制
        int maxRowsPerSheet = 65536; // 对于 .xls 文件，最大行数为 65536
        int batchSize = 1000; // 每次批量处理1000行，减少逐行写入的开销

        // 复制表头到新sheet的逻辑
        ExcelUtils.copyHeader(sheet, style);

        // 批量处理数据
        for (int i = 0; i < records.size(); i += batchSize) {
            // 处理一批数据
            int toIndex = Math.min(i + batchSize, records.size());
            List<HyMtqsEVo> batchList = records.subList(i, toIndex);

            // 批量写入数据
            for (HyMtqsEVo vo : batchList) {
                // 如果当前行数达到最大行数，创建新sheet
                if (rowIndex >= maxRowsPerSheet) {
                    sheetIndex++;
                    sheet = sheet.getWorkbook().createSheet("月水面蒸发量表" + sheetIndex);
                    rowIndex = 2; // 重置行号
                    // 复制表头到新sheet
                    ExcelUtils.copyHeader(sheet, style);
                }

                // 创建新行并写入数据
                Row row = ExcelUtils.createRow(sheet, rowIndex);

                // 使用 globalIndex 作为序号
                ExcelUtils.formatCell(row, 0, String.valueOf(globalIndex), style);
                ExcelUtils.formatCell(row, 1, vo.getStcd() != null ? vo.getStcd() : "", style);
                ExcelUtils.formatCell(row, 2, vo.getStnm() != null ? vo.getStnm() : "", style);
                ExcelUtils.formatCell(row, 3, vo.getAddvcd() != null ? vo.getAddvcd() : "", style);
                ExcelUtils.formatCell(row,4,"",style);
                ExcelUtils.formatCell(row, 5, vo.getYr() != null ? vo.getYr().toString() : "", style);
                ExcelUtils.formatCell(row, 6, vo.getMth() != null ? vo.getMth().toString() : "", style);
                ExcelUtils.formatCell(row, 7, vo.getSdtp() != null ? vo.getSdtp() : "", style);
                ExcelUtils.formatCell(row, 8, vo.getAvqs() != null ? CommonUtils.formatNum(vo.getAvqs()) : "", style);
                ExcelUtils.formatCell(row, 9, vo.getAvqsrcd() != null ? vo.getAvqsrcd() : "", style);
                ExcelUtils.formatCell(row, 10, vo.getMxdyqs() != null ? CommonUtils.formatNum(vo.getMxdyqs()) : "", style);
                ExcelUtils.formatCell(row, 11, vo.getMxdyqsrcd() != null ? vo.getMxdyqsrcd() : "", style);
                ExcelUtils.formatCell(row, 12, vo.getMxdyqsodt() != null ? vo.getMxdyqsodt() : "", style);

                // 增加行号和全局序号
                rowIndex++;
                globalIndex++;
            }
        }
        return 0;
    }

    public void dataModel(Map<String,Object> map, Sheet sheet, CellStyle style) {
        // 批量查询数据
        IPage<HyMtqsEVo> hyMtqsEVoIPage = this.baseMapper.selectHyMtqsEByPageAndInfo(new Query<HyMtqsEVo>().getPage(map), map);
        List<HyMtqsEVo> records = hyMtqsEVoIPage.getRecords();
        // 定义当前sheet的行号
        int rowIndex = 2;
        // 定义当前sheet的编号
        int sheetIndex = 1;
        Row row;
        // 最大行数限制
        int maxRowsPerSheet = 65536; // 对于 .xls 文件，最大行数为 65536
        int batchSize = 1000; // 每次批量处理1000行，减少逐行写入的开销
        // 批量处理数据
        for (int i = 0; i < records.size(); i += batchSize) {
            // 处理一批数据
            int toIndex = Math.min(i + batchSize, records.size());
            List<HyMtqsEVo> batchList = records.subList(i, toIndex);
            // 批量写入数据
            for (HyMtqsEVo vo : batchList) {
                // 如果当前行数达到最大行数，创建新sheet
                if (rowIndex >= maxRowsPerSheet) {
                    sheetIndex++;
                    sheet = sheet.getWorkbook().createSheet("月水面蒸发量表" + sheetIndex);
                    rowIndex = 2; // 重置行号
                }
                // 创建新行并写入数据
                row = ExcelUtils.createRow(sheet, rowIndex);
                //序号
                ExcelUtils.formatCell(row, 0, String.valueOf(rowIndex - 1), style);
                ExcelUtils.formatCell(row, 1, vo.getStcd() != null ? vo.getStcd() : "", style);
                ExcelUtils.formatCell(row, 2, vo.getStnm() != null ? vo.getStnm() : "", style);
                ExcelUtils.formatCell(row, 3, vo.getAddvcd() != null ? vo.getAddvcd() : "", style);
                ExcelUtils.formatCell(row,4,"",style);
                ExcelUtils.formatCell(row, 5, vo.getYr() != null ? vo.getYr().toString() : "", style);
                ExcelUtils.formatCell(row, 6, vo.getMth() != null ? vo.getMth().toString() : "", style);
                ExcelUtils.formatCell(row, 7, vo.getSdtp() != null ? vo.getSdtp() : "", style);
                ExcelUtils.formatCell(row, 8, vo.getAvqs() != null ? CommonUtils.formatNum(vo.getAvqs()) : "", style);
                ExcelUtils.formatCell(row, 9, vo.getAvqsrcd() != null ? vo.getAvqsrcd() : "", style);
                ExcelUtils.formatCell(row, 10, vo.getMxdyqs() != null ? CommonUtils.formatNum(vo.getMxdyqs()) : "", style);
                ExcelUtils.formatCell(row, 11, vo.getMxdyqsrcd() != null ? vo.getMxdyqsrcd() : "", style);
                ExcelUtils.formatCell(row, 12, vo.getMxdyqsodt() != null ? vo.getMxdyqsodt() : "", style);
                rowIndex++;
            }
        }
    }
}




