package com.ruoyi.swlscx.year.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ruoyi.common.core.page.R;
import com.ruoyi.swlscx.common.PageParams;
import com.ruoyi.swlscx.year.domain.vo.*;
import com.ruoyi.swlscx.year.service.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

/**
 * @Author al
 * @Date 2024/7/23 14:04
 * @Description: TODO
 * @Version
 */

@RestController
@RequestMapping("/year")
@RequiredArgsConstructor
@Slf4j
public class YearTableController {

    private final HyHmxpFService hyHmxpFService;

    private final HyYrpFService hyYrpFService;

    private final HyYrweFService hyYrweFService;

    private final HyYrzFService hyYrzFService;

    private final HyYrqFService hyYrqFService;

    private final HyMmxpFService hyMmxpFService;

    private final HyDmxpFService hyDmxpFService;

    private final HyImxfwFService hyImxfwFService;

    private final HyYrcsFService hyYrcsFService;

    private final HyYrqsFService hyYrqsFService;

    private final HyYrwtFService hyYrwtFService;

    private final HyYrtdzFService hyYrtdzFService;

    /**
     * 导入时段最大降雨量数据
     */
    @PostMapping("/importHourMaxRainData")
    public R importHourMaxRainData(MultipartFile file) {
        log.info("file:{}", file);
        hyHmxpFService.importHourMaxRainData(file);
        return R.ok();
    }


    /**
     * 查询年降水表数据
     */
    @GetMapping("/getYearRainList")
    public R getMonthRainList(Integer pageNum, Integer pageSize, String startTime, String endTime, String stcd, String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyYrpFVo> hyYrpFVoIPage = hyYrpFService.selectHyYrpFDataByPageAndInfo(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", hyYrpFVoIPage.getRecords()).put("count", hyYrpFVoIPage.getTotal());
    }

    /**查询年水面蒸发表数据**/
    @RequestMapping("/getEvaporationWaterList")
    public R getEvaporationWaterList(Integer pageNum,Integer pageSize, String startTime, String endTime,String stcd,String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyYrweFVo> hyYrweFVoIPage = hyYrweFService.selectHyYrweFListPageByInfo(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", hyYrweFVoIPage.getRecords()).put("count", hyYrweFVoIPage.getTotal());
    }

    /**查询年水位表数据**/
    @GetMapping("/getYearWaterLeverList")
    public R getWaterLeverList(Integer pageNum,Integer pageSize, String startTime, String endTime,String stnm,String stcd) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyYrzFVo> hyYrzFVoIPage = hyYrzFService.selectHyYrzFPageByInfo(pageParams, startTime, endTime, stnm, stcd);
        return R.ok().put("data", hyYrzFVoIPage.getRecords()).put("count", hyYrzFVoIPage.getTotal());
    }


    /**查询年流量表数据**/
    @GetMapping("/getYearFlowList")
    public R getFlowList(Integer pageNum, Integer pageSize, String startTime, String endTime, String stcd, String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyYrqFVo> hyYrqFVoIPage = hyYrqFService.selecHyYrqFListPageByInfo(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", hyYrqFVoIPage.getRecords()).put("count", hyYrqFVoIPage.getTotal());
    }



    /**
     * 查询分钟时段最大降水量表数据
     */
    @RequestMapping("/getMaxMRainList")
    public R getMaxMRainList(Integer pageNum,Integer pageSize, String startTime, String endTime,String stcd,String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyMmxpFVo> hyMmxpFVoIPage = hyMmxpFService.selectMaxMRainListPageByInfo(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", hyMmxpFVoIPage.getRecords()).put("count", hyMmxpFVoIPage.getTotal());
    }

    /**
     * 查询小时时段最大降水量表数据
     */
    @RequestMapping("/getMaxHRainList")
    public R getMaxHRainList(Integer pageNum,Integer pageSize, String startTime, String endTime,String stcd,String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyHmxpFVo> hyHmxpFVoIPage = hyHmxpFService.selectMaxHRainListPageByInfo(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", hyHmxpFVoIPage.getRecords()).put("count", hyHmxpFVoIPage.getTotal());
    }

    /**
     * 查询日时时段最大降水量表
     */
    @RequestMapping("/getMaxDRainList")
    public R getMaxDRainList(Integer pageNum,Integer pageSize, String startTime, String endTime,String stcd,String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyDmxpFVo> hyDmxpFVoIPage = hyDmxpFService.selectMaxDRainListPageByInfo(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", hyDmxpFVoIPage.getRecords()).put("count", hyDmxpFVoIPage.getTotal());
    }


    /**
     * 查询时段最大洪量表数据
     */
    @RequestMapping("/getMaxFloodScaleList")
    public R getMaxFloodScaleList(Integer pageNum,Integer pageSize, String startTime, String endTime,String stcd,String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyImxfwFVo> hyImxfwFVoIPage = hyImxfwFService.selectMaxFloodScaleListPageByInfo(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", hyImxfwFVoIPage.getRecords()).put("count", hyImxfwFVoIPage.getTotal());
    }


    /**
     * 查询年含沙量表数据
     */
    @RequestMapping("/getYearSedimentConcentrationList")
    public R getYearSedimentConcentrationList(Integer pageNum,Integer pageSize, String startTime, String endTime,String stcd,String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyYrcsFVo> yearSedimentConcentrationList = hyYrcsFService.getYearSedimentConcentrationList(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", yearSedimentConcentrationList.getRecords()).put("count", yearSedimentConcentrationList.getTotal());
    }

    /**
     * 查询年输沙率表数据
     */
    @RequestMapping("/getYearsedimentTransportRateList")
    public R getYearsedimentTransportRateList(Integer pageNum,Integer pageSize, String startTime, String endTime,String stcd,String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyYrqsFVo> yearsedimentTransportRateList = hyYrqsFService.getYearsedimentTransportRateList(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", yearsedimentTransportRateList.getRecords()).put("count", yearsedimentTransportRateList.getTotal());
    }


    /**
     * 查询年水温表数据
     */
    @RequestMapping("/getYearWaterTemperatureList")
    public R getYearWaterTemperatureList(Integer pageNum,Integer pageSize, String startTime, String endTime,String stcd,String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyYrwtFVo> yearWaterTemperatureList = hyYrwtFService.getYearWaterTemperatureList(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", yearWaterTemperatureList.getRecords()).put("count", yearWaterTemperatureList.getTotal());
    }

    /**
     * 查询年潮位表数据
     */
    @RequestMapping("/getYearTideLeverList")
    public R getYearTideLeverList(Integer pageNum,Integer pageSize, String startTime, String endTime,String stcd,String stnm) {
        PageParams pageParams = new PageParams(pageNum.longValue(), pageSize.longValue());
        IPage<HyYrtdzFVo> hyYrtdzFVoIPage = hyYrtdzFService.selectHyYrtdzFListPageByInfo(pageParams, startTime, endTime, stcd, stnm);
        return R.ok().put("data", hyYrtdzFVoIPage.getRecords()).put("count", hyYrtdzFVoIPage.getTotal());
    }

}
